@@ -104,14 +104,17 @@
           *resize-map*
           *default-group-name*
           *window-border-style*
+          *data-dir*
           add-hook
+          clear-window-placement-rules
           dformat
           define-frame-preference
           remove-hook
           run-hook
           run-hook-with-args
           split-string
-	  with-restarts-menu))
+	  with-restarts-menu
+          with-data-file))
 
 
 ;;; Message Timer
@@ -523,6 +526,9 @@ single char keys are supported.")
 (defvar *executing-stumpwm-command* nil
   "True when executing external commands.")
 
+(defvar *interactivep* nil
+  "True when a defcommand is executed from colon or a keybinding")
+ 
 ;;; The restarts menu macro
 
 (defmacro with-restarts-menu (&body body)
@@ -554,11 +560,19 @@ chosen, resignal the error."
   (run-hook-with-args hook))
 
 (defmacro add-hook (hook fn)
-  "Add a function to a hook."
+  "Add @var{function} to the hook @var{hook-variable}. For example, to
+display a message whenever you switch frames:
+
+@example
+\(defun my-rad-fn (from-frame to-frame)
+  (stumpwm:message \"Mustard!\"))
+
+\(stumpmwm:add-hook stumpwm:*focus-frame-hook* 'my-rad-fn)
+@end example"
   `(setf ,hook (adjoin ,fn ,hook)))
 
 (defmacro remove-hook (hook fn)
-  "Remove a function from a hook."
+"Remove the specified function from the hook."
   `(setf ,hook (remove ,fn ,hook)))
 
 ;; Misc. utility functions
@@ -898,7 +912,7 @@ window, and returns the preferred frame or a list of the above preferences.")
   "This is the message StumpWM displays when it starts. Set it to NIL to
 suppress.")
 
-(defvar *default-package* (find-package "CL-USER")
+(defvar *default-package* (find-package '#:stumpwm-user)
   "This is the package eval reads and executes in. You might want to set
 this to @code{:stumpwm} if you find yourself using a lot of internal
 stumpwm symbols. Setting this variable anywhere but in your rc file
@@ -912,14 +926,50 @@ will have no effect.")
 add rules")
 
 
-(defmacro define-frame-preference (group &rest frames)
+(defmacro define-frame-preference (target-group &rest frame-rules)
+  "Create a rule that matches windows and automatically places them in
+a specified group and frame. Each frame rule is a lambda list:
+@example
+\(frame-number raise lock &key class instance type role title)
+@end example
+
+@table @var
+@item frame-number
+The frame number to send matching windows to
+
+@item raise
+When non-nil, raise and focus the window in its frame
+
+@item lock
+When this is nil, this rule will only match when the current group
+matches @var{target-group}. When non-nil, this rule matches regardless
+of the group and the window is sent to @var{target-group}. If
+@var{lock} and @var{raise} are both non-nil, then stumpwm will jump to
+the specified group and focus the matched window.
+
+@item class
+The window's class must match @var{class}.
+
+@item instance
+The window's instance/resource name must match @var{instance}.
+
+@item type
+The window's type must match @var{type}.
+
+@item role
+The window's role must match @var{role}.
+
+@item title
+The window's title must match @var{title}.
+@end table"
   (let ((x (gensym "X")))
-    `(dolist (,x ',frames)
+    `(dolist (,x ',frame-rules)
        ;; verify the correct structure
        (destructuring-bind (frame-number raise lock &rest keys &key class instance type role title) ,x
-         (push (list* ,group frame-number raise lock keys) *window-placement-rules*)))))
+         (push (list* ,target-group frame-number raise lock keys) *window-placement-rules*)))))
 
 (defun clear-window-placement-rules ()
+  "Clear all window placement rules."
   (setf *window-placement-rules* nil))
 
 (defvar *mouse-focus-policy* :ignore
@@ -987,3 +1037,17 @@ Like :tight but no border is ever visible.
 
 After changing this variable you may need to call
 sync-all-frame-windows to see the change.")
+
+(defvar *data-dir* (make-pathname :directory (append (pathname-directory (user-homedir-pathname))
+                                                     (list ".stumpwm.d")))
+  "The directory used by stumpwm to store data between sessions.")
+
+(defmacro with-data-file ((s file &rest keys &key (if-exists :supersede) &allow-other-keys) &body body)
+  "Open a file in StumpWM's data directory. keyword arguments are sent
+directly to OPEN. Note that IF-EXISTS defaults to :supersede, instead
+of :error."
+  `(progn
+     (ensure-directories-exist *data-dir*)
+     (with-open-file (,s ,(merge-pathnames *data-dir* file)
+                         ,@keys)
+       ,@body)))
